/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package bin.tutorial2;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;
import org.biojava3.alignment.Alignments;
import org.biojava3.alignment.Alignments.PairwiseSequenceAlignerType;
import org.biojava3.alignment.SimpleGapPenalty;
import org.biojava3.alignment.SubstitutionMatrixHelper;
import org.biojava3.alignment.template.SequencePair;
import org.biojava3.alignment.template.SubstitutionMatrix;
import org.biojava3.core.sequence.DNASequence;
import org.biojava3.core.sequence.ProteinSequence;
import org.biojava3.core.sequence.compound.AminoAcidCompound;
import org.biojava3.core.sequence.compound.NucleotideCompound;
import org.biojava3.core.sequence.io.FastaReaderHelper;
import org.biojava3.core.sequence.template.Sequence;
import org.biojava3.core.sequence.transcription.TranscriptionEngine;

/**
 *
 * @author Ondra
 */
public class PairwiseAlignment {
    
    public static void printGlobalNucleotideAlignment_AminoAcid(String fasta1, String fasta2, int start1, int end1, int start2, int end2) throws Exception {
        //first, we read the nucleotide sequences from two files
        LinkedHashMap<String, DNASequence> a = FastaReaderHelper.readFastaDNASequence(new File(fasta1));
        LinkedHashMap<String, DNASequence> b = FastaReaderHelper.readFastaDNASequence(new File(fasta2));
        //now, we obtain one sequence from each of the HashMaps - in general, there may be more than one sequence in
        //a fasta file
        ProteinSequence target = null;
        for (Map.Entry<String,DNASequence> entry : a.entrySet()){
            //we get the default transcription engine - transcription is meant in the biological sense
            TranscriptionEngine engine = TranscriptionEngine.getDefault(); //Get the default engine
            //we obtain RNA sequence corresponding to the selected region (from start1 to end1) of the sequence in the first fasta file
            Sequence<NucleotideCompound> rna = engine.getDnaRnaTranslator().createSequence(new DNASequence(entry.getValue().toString().substring(start1-1, end1)));
            //we translate the RNA to a sequence of amino acids (this is done on ribosomes in cells :-))
            Sequence<AminoAcidCompound> protein = engine.getRnaAminoAcidTranslator().createSequence(rna);
            target = new ProteinSequence(protein.toString());
            break;//we expect just one sequence in the file
        }
        ProteinSequence query = null;
        //the same as in the previous block of code, just with the second file this time
        for (Map.Entry<String,DNASequence> entry : b.entrySet()){
            TranscriptionEngine engine = TranscriptionEngine.getDefault(); //Get the default engine
            Sequence<NucleotideCompound> rna = engine.getDnaRnaTranslator().createSequence(new DNASequence(entry.getValue().toString().substring(start2-1, end2)));
            Sequence<AminoAcidCompound> protein = engine.getRnaAminoAcidTranslator().createSequence(rna);
            query = new ProteinSequence(protein.toString());
            break;//we expect just one sequence in the file
        }
        
        //the substitution matrix containing penalties for "substitution" of amino acids
        SubstitutionMatrix<AminoAcidCompound> matrix = SubstitutionMatrixHelper.getBlosum60();
        
        //setting the penalty for opening (5) and extending (2) a gap in the alignment
        SimpleGapPenalty gapP = new SimpleGapPenalty();
        gapP.setOpenPenalty((short)5);
        gapP.setExtensionPenalty((short)2);

        //computing the pairwise sequence alignment - we can have either global (PairwiseSequenceAlignerType.GLOBAL)
        //or local (PairwiseSequenceAlignerType.LOCAL) alignments
        SequencePair<ProteinSequence, AminoAcidCompound> psa = Alignments.getPairwiseAlignment(query, target, 
                PairwiseSequenceAlignerType.GLOBAL, gapP, matrix);
        System.out.println(psa);
    }
    
    public static void printGlobalNucleotideAlignment_DNA(String fasta1, String fasta2) throws Exception {
        //first, we read the nucleotide sequences from two files
        LinkedHashMap<String, DNASequence> a = FastaReaderHelper.readFastaDNASequence(new File(fasta1));
        LinkedHashMap<String, DNASequence> b = FastaReaderHelper.readFastaDNASequence(new File(fasta2));
        //now, we obtain one sequence from each of the HashMaps - in general, there may be more than one sequence in
        //a fasta file
        DNASequence target = null;
        for (Map.Entry<String,DNASequence> entry : a.entrySet()){
            target = entry.getValue();
            break;//we expect just one sequence in the file
        }
        //the same as in the previous block of code, just with the second file this time
        DNASequence query = null;
        for (Map.Entry<String,DNASequence> entry : b.entrySet()){
            query = entry.getValue();
            break;//we expect just one sequence in the file
        }
        
        //substitution matrix for nucleotides
        SubstitutionMatrix<NucleotideCompound> matrix = SubstitutionMatrixHelper.getNuc4_4();
        //setting the penalty for opening (5) and extending (2) a gap in the alignment
        SimpleGapPenalty gapP = new SimpleGapPenalty();
        gapP.setOpenPenalty((short)5);
        gapP.setExtensionPenalty((short)2);
        
        //computing the pairwise sequence alignment - we can have either global (PairwiseSequenceAlignerType.GLOBAL)
        //or local (PairwiseSequenceAlignerType.LOCAL) alignments
        SequencePair<DNASequence, NucleotideCompound> psa = Alignments.getPairwiseAlignment(query, target, PairwiseSequenceAlignerType.GLOBAL, gapP, matrix);
        System.out.println(psa);
    }
    
    public static void main(String[] args) throws Exception {
        if (args.length < 3){
            System.out.println("Arguments: paths to the two FASTA files and a flag: aminoacid or dna!");
            System.exit(0);
        }
        String filename1 = args[0];
        String filename2 = args[1];
        String flag = args[2];
        
        if (flag.equalsIgnoreCase("aminoacid")){
            printGlobalNucleotideAlignment_AminoAcid(filename1, filename2,
                    Integer.parseInt(args[3]), Integer.parseInt(args[4]), Integer.parseInt(args[5]), Integer.parseInt(args[6]));
        } else if (flag.equalsIgnoreCase("dna")){
            printGlobalNucleotideAlignment_DNA(filename1, filename2);
        } else {
            System.out.println("Unrecognized option: "+flag);
            System.exit(0);
        }
    }
    
}
